# This file was auto-generated by Fern from our API Definition.

import typing

ProviderApiKeyPublicProvider = typing.Union[
    typing.Literal["openai", "anthropic", "gemini", "openrouter", "vertex-ai", "custom-llm"], typing.Any
]
